# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Autopilot Functional Test Tool
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest.mock import (
    Mock,
    patch
)
from testtools import TestCase

from autopilot.introspection import _proxy_objects as _po


class GetProxyObjectClassesTests(TestCase):
    pass


class IsCpoClassTests(TestCase):

    def test_returns_False_when_None(self):
        self.assertFalse(_po._is_cpo_class(None))

    def test_returns_False_when_is_string(self):
        self.assertFalse(_po._is_cpo_class('foo'))

    def test_returns_True_when_is_class(self):
        class Foo():
            pass
        self.assertTrue(_po._is_cpo_class(Foo))


class GetApplicableClassNameTests(TestCase):

    @patch.object(_po, '_get_proxy_object_class_name_and_state')
    def test_queries_dbus_name_unknown(self, po_cs):
        fake_dbus_backend = Mock()
        po_cs.return_value = (None, None, None)
        _po._get_applicable_class_name(None, fake_dbus_backend)
        po_cs.assert_called_once_with(fake_dbus_backend)

    @patch.object(_po, '_get_proxy_object_class_name_and_state')
    def test_returns_dbus_name_when_passed_None(self, po_cs):
        name = self.getUniqueString()
        po_cs.return_value = (name, None, None)
        fake_dbus_backend = Mock()

        self.assertEqual(
            name,
            _po._get_applicable_class_name(
                None,
                fake_dbus_backend
            )
        )

    @patch.object(_po, '_get_proxy_object_class_name_and_state')
    def test_returns_dbus_name_when_passed_star(self, po_cs):
        name = self.getUniqueString()
        po_cs.return_value = (name, None, None)
        fake_dbus_backend = Mock()

        self.assertEqual(
            name,
            _po._get_applicable_class_name(
                '*',
                fake_dbus_backend
            )
        )

    def test_returns_string_from_classname(self):
        name = self.getUniqueString()
        fake_dbus_backend = Mock()
        with patch.object(_po, 'get_classname_from_path') as from_path:
            from_path.return_value = bytes(name, 'utf-8')
            self.assertEqual(
                _po._get_applicable_class_name('*', fake_dbus_backend),
                name
            )

    def test_returns_name_of_string(self):
        name = self.getUniqueString()
        self.assertEqual(name, _po._get_applicable_class_name(name, None))
